/* TEMPLATE GENERATED TESTCASE FILE
Filename: CWE123_Write_What_Where_Condition__connect_socket_31.c
Label Definition File: CWE123_Write_What_Where_Condition.label.xml
Template File: sources-sink-31.tmpl.c
*/
/*
 * @description
 * CWE: 123 Write-What-Where Condition
 * BadSource: connect_socket Overwrite linked list pointers using a connect socket (client side)
 * GoodSource: Don't overwrite linked list pointers
 * Sinks:
 *    BadSink : Remove element from list
 * Flow Variant: 31 Data flow using a copy of data within the same function
 *
 * */

#include "std_testcase.h"

typedef struct _linked_list
{
    struct _linked_list *next;
    struct _linked_list *prev;
} linked_list;

typedef struct _bad_struct
{
    linked_list list;
} bad_struct;

static linked_list *prev, *next;

#ifdef _WIN32
# include <winsock2.h>
# include <windows.h>
# include <direct.h>
# pragma comment(lib, "ws2_32") /* include ws2_32.lib when linking */
# define CLOSE_SOCKET closesocket
#else /* NOT _WIN32 */
# define INVALID_SOCKET -1
# define SOCKET_ERROR -1
# define CLOSE_SOCKET close
# define SOCKET int
#endif

#define TCP_PORT 27015

#ifndef OMITBAD

void CWE123_Write_What_Where_Condition__connect_socket_31_bad()
{
    bad_struct data;
    linked_list head = { &head, &head };
    /* This simulates a Microsoft-style linked list insertion */
    data.list.next = head.next;
    data.list.prev = &head;
    head.next = &data.list;
    head.next->prev = &data.list;
    {
#ifdef _WIN32
        WSADATA wsa_data;
        int wsa_data_init = 0;
#endif
        int recv_rv;
        struct sockaddr_in s_in;
        SOCKET connect_socket = INVALID_SOCKET;
        do
        {
#ifdef _WIN32
            if (WSAStartup(MAKEWORD(2,2), &wsa_data) != NO_ERROR) break;
            wsa_data_init = 1;
#endif
            connect_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
            if (connect_socket == INVALID_SOCKET) break;
            memset(&s_in, 0, sizeof(s_in));
            s_in.sin_family = AF_INET;
            s_in.sin_addr.s_addr = inet_addr("127.0.0.1");
            s_in.sin_port = htons(TCP_PORT);
            if (connect(connect_socket, (struct sockaddr*)&s_in, sizeof(s_in)) == SOCKET_ERROR) break;
            /* Abort on error or the connection was closed, make sure to recv one
             * less char than is in the recv_buf in order to append a terminator */
            /* FLAW: overwrite linked list pointers with data */
            recv_rv = recv(connect_socket, (char*)&data, sizeof(data), 0);
            if (recv_rv == SOCKET_ERROR || recv_rv == 0) break;
        }
        while (0);
        if (connect_socket != INVALID_SOCKET) CLOSE_SOCKET(connect_socket);
#ifdef _WIN32
        if (wsa_data_init) WSACleanup();
#endif
    }
    {
        bad_struct data_copy = data;
        bad_struct data = data_copy;
        /* POTENTIAL FLAW: The following removes 'a' from the list.  Because of the possible overflow this
         * causes a "write-what-where" aka "write4".  It does another write as
         * well.  But this is the prototypical "write-what-where" at least from
         * the Windows perspective.
         *
         * prev = a->list->prev  WHAT
         * next = a->list->next  WHERE
         * prev->next = next  "at the address that prev/WHERE points, write
         *                    next/WHAT"
         *                    aka "write-what-where"
         * next->prev = prev  "at the address that next/WHAT points plus 4
         *                    (because prev is the second field in 'list' hence
         *                    4 bytes away on 32b machines), write prev/WHERE"
         */
        prev = data.list.prev;
        next = data.list.next;
        prev->next = next;
        next->prev = prev;
    }
}

#endif /* OMITBAD */

#ifndef OMITGOOD

/* goodG2B() uses the GoodSource with the BadSink */
static void goodG2B()
{
    bad_struct data;
    linked_list head = { &head, &head };
    /* This simulates a Microsoft-style linked list insertion */
    data.list.next = head.next;
    data.list.prev = &head;
    head.next = &data.list;
    head.next->prev = &data.list;
    /* FIX: don't overwrite linked list pointers */
    ; /* empty statement needed by some flow variants */
    {
        bad_struct data_copy = data;
        bad_struct data = data_copy;
        /* POTENTIAL FLAW: The following removes 'a' from the list.  Because of the possible overflow this
         * causes a "write-what-where" aka "write4".  It does another write as
         * well.  But this is the prototypical "write-what-where" at least from
         * the Windows perspective.
         *
         * prev = a->list->prev  WHAT
         * next = a->list->next  WHERE
         * prev->next = next  "at the address that prev/WHERE points, write
         *                    next/WHAT"
         *                    aka "write-what-where"
         * next->prev = prev  "at the address that next/WHAT points plus 4
         *                    (because prev is the second field in 'list' hence
         *                    4 bytes away on 32b machines), write prev/WHERE"
         */
        prev = data.list.prev;
        next = data.list.next;
        prev->next = next;
        next->prev = prev;
    }
}

void CWE123_Write_What_Where_Condition__connect_socket_31_good()
{
    goodG2B();
}

#endif /* OMITGOOD */

/* Below is the main(). It is only used when building this testcase on
   its own for testing or for building a binary to use in testing binary
   analysis tools. It is not used when compiling all the testcases as one
   application, which is how source code analysis tools are tested. */
#ifdef INCLUDEMAIN

int main(int argc, char * argv[])
{
    /* seed randomness */
    srand( (unsigned)time(NULL) );
#ifndef OMITGOOD
    printLine("Calling good()...");
    CWE123_Write_What_Where_Condition__connect_socket_31_good();
    printLine("Finished good()");
#endif /* OMITGOOD */
#ifndef OMITBAD
    printLine("Calling bad()...");
    CWE123_Write_What_Where_Condition__connect_socket_31_bad();
    printLine("Finished bad()");
#endif /* OMITBAD */
    return 0;
}

#endif
